#include <stdio.h>
#include <stdlib.h>
#include <string.h>

struct punto2D {
	int x, y;
};

typedef struct punto2D PUNTO2D;

PUNTO2D * p0;

// 1. Cuenta el número de puntos en el archivo "nombre"
// 2. Reserva la memoria para almacenar n puntos
// 3. Lee de nuevo el archivo para guardar los valores en el 
//      arreglo
PUNTO2D * lee_puntos( char * nombre, int *pn )
{
	FILE * arch;
	char linea[80], *paux;
	int v1, v2, i, n;
	PUNTO2D *pts;

	arch = fopen( nombre, "r" );
	if( arch == NULL ) {
		fprintf( stderr, "No se puede abrir el archivo %s\n", nombre );
		return NULL;
	}

	n = 0;
	while( 1 ) {
		if( fgets( linea, 80, arch ) == NULL ) break;
		paux = linea;
		while ( *paux == ' ' || *paux == '\t' )
			paux++;

		i = sscanf( paux, "%d %d\n", &v1, &v2 );
		if ( i == 2 ) 
			n++;
	}
	// El número de puntos está en "n"
	*pn = n;
	pts = (PUNTO2D *)malloc( n*sizeof(PUNTO2D) );
	if ( pts == NULL ) {
		fprintf( stderr, "No hay memoria\n" );
		return NULL;
	}

	// Leo el archivo de nuevo
	fseek( arch, 0, SEEK_SET );
	n = 0;
	while( 1 ) {
		if( fgets( linea, 80, arch ) == NULL ) break;
		paux = linea;
		while ( *paux == ' ' || *paux == '\t' )
			paux++;

		i = sscanf( paux, "%d %d\n", &v1, &v2 );
		if ( i == 2 ) {
			pts[n].x = v1;
			pts[n].y = v2;
			n++;
		} 
	}
	
	return pts;
}

int compara( const void *p1, const void *p2 )
{
	PUNTO2D *pa, *pb;
	PUNTO2D v1, v2;
	int area2, d1, d2;

	pa = (PUNTO2D *)p1;
	pb = (PUNTO2D *)p2;

	v1.x = pa->x - p0->x;
	v1.y = pa->y - p0->y;

	v2.x = pb->x - p0->x;
	v2.y = pb->y - p0->y;

	area2 = v1.x * v2.y - v1.y * v2.x;

	if( area2 > 0 ) return -1;
	else if (area2 < 0 ) return 1;
	else {
		d1 = v1.x * v1.x + v1.y * v1.y;
		d2 = v2.x * v2.x + v2.y * v2.y;

		if ( d1 < d2 ) return -1;
		else return 1;
	}
}


void ordenamientoCircular( PUNTO2D * ptos, int n )
{
	int i, minimo;

	// Busco el punto con y mínima, si hay varios de ellos
	// encuentro el punto con x mínima:
	minimo = 0;

	for( i=1; i<n; i++ ) {
		if( ptos[i].y < ptos[minimo].y )
			minimo = i;
		else if( ptos[i].y == ptos[minimo].y && ptos[i].x < ptos[minimo].x )
			minimo = i;
	}

	// Intercambio el punto[minimo] con punto[0]
	i = ptos[0].x;
	ptos[0].x = ptos[minimo].x;
	ptos[minimo].x = i;
	
	i = ptos[0].y;
	ptos[0].y = ptos[minimo].y;
	ptos[minimo].y = i;

	p0 = ptos;

	// Ordenamos los puntos!
	qsort( (void *)&ptos[1], n-1, sizeof(PUNTO2D), compara );
}


int main( int argc, char *argv[] )
{
	PUNTO2D *puntos;
	int i, n;
	char nombre[32];

	if( argc != 2 ) {
		fprintf( stderr, "Args: archivo_entrada\n" );
		return 1;
	}
	strncpy( nombre, argv[1], 32 );

	puntos = lee_puntos( nombre, &n );

	if ( puntos != NULL ) {
		ordenamientoCircular( puntos, n );

		for(i=0; i<n; i++) 
			printf( "%d %d\n", puntos[i].x, puntos[i].y );

		free( puntos );
		printf( "# Ok!\n" );
	}

	return 0;
}
